// TaskBarCmd.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

// This was introduced in Dev.Studio 2005.
// If we don't have it, we declare a dumb-down version
#ifndef _countof
#define _countof(a) (sizeof(a)/sizeof(a[0]))
#endif // _countof

#define CMD_AUTOHIDE	1
#define CMD_NOAUTOHIDE	2

#define CMD_KEEPONTOP	1
#define CMD_NOKEEPONTOP	2

#define CMD_QLAUNCHSHOW		1
#define CMD_NOQLAUNCHSHOW	2

//======== Quick Launch part ========

// Undocumented, wild guess, might break at anytime!
// If the program stops working, this is the first one to check!
#define	QUICKLAUNCH_BANDID 10

bool QuickLaunch( int cmdShow ) {
	HRESULT hRez;
	bool nResult = false;

	hRez = CoInitialize( NULL );
	if( hRez != S_OK )
		return nResult;

	IBandSite	*pBandSite;
	hRez = CoCreateInstance( CLSID_TrayBandSiteService, NULL, CLSCTX_SERVER, IID_IBandSite, (LPVOID*)&pBandSite );
	if( pBandSite ) {
		IDeskBand	*pBand;
		TCHAR bufName[1024];
		DWORD dwState, dwOrgState;

		hRez = pBandSite->QueryBand( QUICKLAUNCH_BANDID, &pBand, &dwOrgState, bufName, _countof(bufName) );
		if( hRez == S_OK ) {
			switch( cmdShow ) {
				case CMD_QLAUNCHSHOW:
					dwState =  dwOrgState | BSSF_VISIBLE;
					break;
				case CMD_NOQLAUNCHSHOW:
					dwState =  dwOrgState & ~BSSF_VISIBLE;
					break;
			}
			if( dwState != dwOrgState ) {
				pBandSite->SetBandState( QUICKLAUNCH_BANDID, BSIM_STATE, dwState );
				nResult = true;
			}
		}
		pBandSite->Release();
	}

	CoUninitialize();

	return nResult;
}

//======== Taskbar part ========

#define TBAR_AUTOHIDE	1 
#define TBAR_KEEPONTOP	2

bool TaskBar( int cmdAutoHide, int cmdKeepOnTop ) {
	APPBARDATA	apd = {0};

	apd.cbSize = sizeof(apd);
	apd.hWnd = FindWindow( _T("Shell_TrayWnd"), NULL );

	LPARAM orgState = SHAppBarMessage( ABM_GETSTATE, &apd );

	apd.lParam = orgState;
	switch( cmdAutoHide ) {
		case CMD_AUTOHIDE:
			apd.lParam |= TBAR_AUTOHIDE;
			break;
		case CMD_NOAUTOHIDE:
			apd.lParam &= ~TBAR_AUTOHIDE;
			break;
	}
	switch( cmdKeepOnTop ) {
		case CMD_KEEPONTOP:
			apd.lParam |= TBAR_KEEPONTOP;
			break;
		case CMD_NOKEEPONTOP:
			apd.lParam &= ~TBAR_KEEPONTOP;
			break;
	}
	if( orgState != apd.lParam ) {
		SHAppBarMessage( ABM_SETSTATE, &apd );
		return true;
	}

	return false;
}

//======== Command line part ========

enum TBarCommand {
	tb_Unknown,
	tb_AutoHide,
	tb_NoAutoHide,
	tb_KeepOnTop,
	tb_NoKeepOnTop,
	tb_ShowQuickLunch,
	tb_NoShowQuickLunch
};

typedef struct Command {
	TBarCommand nCmd;
	TCHAR *szFlag;
} COMMAND;

COMMAND allCommands[] = {
	{ tb_AutoHide,   _T("-AutoHide") },
	{ tb_AutoHide,   _T("+AH") },
	{ tb_NoAutoHide, _T("-NoAutoHide") },
	{ tb_NoAutoHide, _T("-AH") },

	{ tb_KeepOnTop,    _T("-KeepOnTop") },
	{ tb_KeepOnTop,    _T("+Top") },
	{ tb_NoKeepOnTop,  _T("-NoKeepOnTop") },
	{ tb_NoKeepOnTop,  _T("-Top") },

	{ tb_ShowQuickLunch,   _T("-QuickLaunch") },
	{ tb_ShowQuickLunch,   _T("+QL") },
	{ tb_NoShowQuickLunch, _T("-NoQuickLaunch") },
	{ tb_NoShowQuickLunch, _T("-QL") }
};

TBarCommand GetBarCommand( TCHAR *szCmd ) {

	if( ! szCmd )
		return tb_Unknown;

	for( int i = 0; i < _countof(allCommands); i++ )
		if( !_tcsicmp( allCommands[i].szFlag, szCmd ) )
			return allCommands[i].nCmd;

	return tb_Unknown;
}

void ShowHelpAndExit( void ) {
	_tprintf(
		_T("TaskBarCmd 1.1; (c) Mihai Nita, Feb. 2006\n")
		_T("Usage: TaskBarCmd [option] ...\n")
		_T("  Options:\n")
		_T("    +AH  or -AutoHide      : Auto-hide the taskbar\n")
		_T("    -AH  or -NoAutoHide    : Don't auto-hide the taskbar\n")
		_T("    +Top or -KeepOnTop     : Keep taskbar on top\n")
		_T("    -Top or -NoKeepOnTop   : Don't keep taskbar on top\n")
		_T("    +QL  or -QuickLaunch   : Show the 'Quick Launch' toolbar\n")
		_T("    -QL  or -NoQuickLaunch : Hide the 'Quick Launch' toolbar\n")
		_T("  Example: TaskBarCmd +AH -Top +QL\n")
		_T("    - will enable autohide\n")
		_T("    - will prevent the toolbar from staying on top\n")
		_T("    - will show Quick Launch\n")
	);
	exit( 1 );
}

int _tmain( int argc, _TCHAR* argv[] ) {
	int cmdTaskbarHide, cmdTaskbarTop, cmdQuickLunch;
	cmdTaskbarHide = cmdTaskbarTop = cmdQuickLunch = 0;

	if( argc == 1 )
		ShowHelpAndExit();
	for( int i = 1; i < argc; i++ )
		switch( GetBarCommand( argv[i] ) ) {
			case tb_AutoHide:
				cmdTaskbarHide = CMD_AUTOHIDE;
				break;
			case tb_NoAutoHide:
				cmdTaskbarHide = CMD_NOAUTOHIDE;
				break;
			case tb_KeepOnTop:
				cmdTaskbarTop = CMD_KEEPONTOP;
				break;
			case tb_NoKeepOnTop:
				cmdTaskbarTop = CMD_NOKEEPONTOP;
				break;
			case tb_ShowQuickLunch:
				cmdQuickLunch = CMD_QLAUNCHSHOW;
				break;
			case tb_NoShowQuickLunch:
				cmdQuickLunch = CMD_NOQLAUNCHSHOW;
				break;
			default:
				ShowHelpAndExit();
		}

	TaskBar( cmdTaskbarHide, cmdTaskbarTop );
	QuickLaunch( cmdQuickLunch );

	return 0;
}
